@@ -17,6 +17,7 @@ Demos
 
    adiabatic_qc_3sat
    bb84
+   bernstein_vazirani
    grover_search
    markov_decoherence
    nmr_sequences
@@ -40,7 +41,7 @@ General-purpose quantum algorithms
    phase_estimation
    find_order
 """
-# Ville Bergholm 2011-2012
+# Ville Bergholm 2011-2014
 
 from __future__ import division, absolute_import, print_function, unicode_literals
 
@@ -49,18 +50,20 @@ from operator import mod
 from copy import deepcopy
 
 import numpy as np
-from numpy import (asarray, array, diag, kron, prod, floor, ceil, sqrt, log2, exp, angle, arange, linspace, logical_not,
-                   sin, cos, arctan2, empty, zeros, ones, eye, sort, nonzero, pi, trace, dot, meshgrid, r_)
+from numpy import (asarray, array, diag, kron, prod, floor, ceil, sqrt, log2, exp, angle, arange, linspace,
+    logical_not, sin, cos, arctan2, empty, zeros, ones, eye, sort, nonzero, pi, trace, dot, meshgrid, r_)
 from numpy.linalg import eig, norm
 import numpy.random as npr
 from scipy.misc import factorial
 import matplotlib.pyplot as plt
 from matplotlib.gridspec import GridSpec
+from mpl_toolkits.mplot3d import Axes3D   # FIXME makes buggy 3d plotting work for some reason
 
-from .base import *
-from .lmap import *
-from .utils import *
-from .state import *
+
+from .base import sx, sy, sz, p0, p1, tol
+from .lmap import lmap, tensor, numstr_to_array
+from .utils import gcd, lcm, qubits, R_y, rand_U, rand_SU, mkron, boson_ladder
+from .state import state, fidelity
 from . import gate, ho, plot
 
 
@@ -89,7 +92,7 @@ def adiabatic_qc_3sat(n=6, n_clauses=None, clauses=None, problem='3sat'):
     """Adiabatic quantum computing demo.
 
     This example solves random 3-SAT problems by simulating the
-    adiabatic quantum algorithm of [Farhi]_.
+    adiabatic quantum algorithm of :cite:`Farhi`.
 
     Note that this is incredibly inefficient because we first essentially
     solve the NP-complete problem classically using an exhaustive
@@ -114,7 +117,7 @@ def adiabatic_qc_3sat(n=6, n_clauses=None, clauses=None, problem='3sat'):
         # generate clauses
         clauses = zeros((n_clauses, 3), int)
         for k in range(n_clauses):
-            bits = range(n)
+            bits = list(range(n))
             for j in range(3):
                 clauses[k, j] = bits.pop(npr.randint(len(bits))) + 1 # zero can't be negated, add one
         clauses = sort(clauses, 1)
@@ -162,7 +165,7 @@ def adiabatic_qc_3sat(n=6, n_clauses=None, clauses=None, problem='3sat'):
     print(clauses)
 
     # build initial Hamiltonian
-    xb = 0.5 * (I - sx)
+    xb = 0.5 * (eye(2) -sx)
     H0 = 0
     for k in range(n):
         H0 += mkron(eye(2 ** k), xb, eye(2 ** (n-k-1)))
@@ -181,9 +184,7 @@ def adiabatic_qc(H0, H1, s0, tmax=50):
     """Adiabatic quantum computing.
 
     This is a helper function for simulating the adiabatic quantum
-    algorithm of [Farhi]_ and plotting the results.
-
-    .. [Farhi] E.Farhi et al., "Quantum Computation by Adiabatic Evolution", arXiv.org:quant-ph/0001106.
+    algorithm of :cite:`Farhi` and plotting the results.
     """
     # Ville Bergholm 2009-2011
 
@@ -228,6 +229,7 @@ def bb84(n=50):
 
     """
     # Ville Bergholm 2009-2012
+    from .base import H
 
     print('\n\n=== BB84 protocol ===\n')
     print('Using {0} transmitted qubits, intercept-resend attack.\n'.format(n))
@@ -273,7 +275,7 @@ and randomly prepares the qubit in either the |0> or the |1> state in that basis
         if basis_E[k]:  q = q.u_propagate(H)
 
         # Eve measures in the basis she has chosen
-        p, res, q = q.measure(do = 'C')
+        _, res, q = q.measure(do = 'C')
         eavesdrop[k] = res
 
         # Eve tries to reverse the changes she made...
@@ -286,7 +288,7 @@ and randomly prepares the qubit in either the |0> or the |1> state in that basis
         if basis_B[k]:  q = q.u_propagate(H)
 
         # Bob measures in the basis he has chosen, and discards the qubit.
-        p, res = q.measure()
+        _, res = q.measure()
         received[k] = res
 
     #sum(xor(sent, eavesdrop))/n
@@ -326,15 +328,12 @@ Eve's presence with the probability 1-(3/4)^k.""")
 
 
 def bernstein_vazirani(n=6, linear=True):
-    """Bernstein-Vazirani algorithm demo.
+    r"""Bernstein-Vazirani algorithm demo.
 
-    Simulates the Bernstein-Vazirani algorithm [BV]_, which, given a black box oracle
+    Simulates the Bernstein-Vazirani algorithm :cite:`BV`, which, given a black box oracle
     implementing a linear Boolean function :math:`f_a(x) := a \cdot x`, returns the bit
     vector a (and thus identifies the function) with just a single oracle call.
-
     If the oracle function is not linear, the algorithm will fail.
-
-    .. [BV] E.Bernstein and U.Vazirani, Proceedings of the 25th Annual ACM Symposium on the Theory of Computing (ACM Press, New York, 1993), pp. 11-20.
     """
     # Ville Bergholm 2011-2012
 
@@ -350,7 +349,7 @@ def bernstein_vazirani(n=6, linear=True):
         return (-1) ** f.reshape((-1, 1))
 
     def linear_func(a):
-        """Builds the linear Boolean function f(x) = a \cdot x as a truth table."""
+        r"""Builds the linear Boolean function f(x) = a \cdot x as a truth table."""
         dim = qubits(len(a))
         N = prod(dim)
         U = empty(N, dtype = int)
@@ -408,10 +407,8 @@ def bernstein_vazirani(n=6, linear=True):
 def grover_search(n=8):
     """Grover search algorithm demo.
 
-    Simulate the Grover search algorithm [Grover]_ formulated using amplitude amplification
+    Simulate the Grover search algorithm :cite:`Grover` formulated using amplitude amplification
     in a system of n qubits.
-
-    .. [Grover] L.K. Grover, "Quantum Computers Can Search Rapidly by Using Almost Any Transformation", PRL 80, 4329 (1998). doi:10.1103/PhysRevLett.80.4329.   
     """
     # Ville Bergholm 2009-2010
 
@@ -429,11 +426,11 @@ def grover_search(n=8):
 
     # black box oracle capable of recognizing the correct answer (given as the diagonal)
     # TODO an oracle that actually checks the solutions by computing (using ancillas?)
-    U_oracle = ones((N,1))
-    U_oracle[sol,0] = -1
+    U_oracle = ones((N, 1))
+    U_oracle[sol, 0] = -1
 
-    U_zeroflip = ones((N,1))
-    U_zeroflip[0,0] = -1
+    U_zeroflip = ones((N, 1))
+    U_zeroflip[0, 0] = -1
 
     s = state(0, qubits(n))
 
@@ -441,7 +438,7 @@ def grover_search(n=8):
     s = s.u_propagate(A)
 
     # Grover iteration
-    for k in range(reps):
+    for _ in range(reps):
         # oracle phase flip
         s.data = -U_oracle * s.data
 
@@ -517,7 +514,7 @@ def nmr_sequences(seqs=None, titles=None):
     Plots the fidelity of each control sequence as a function
     of both off-resonance error f and fractional pulse length error g.
 
-    Reproduces fidelity plots in [Cummins]_.
+    Reproduces fidelity plots in :cite:`Cummins`.
     """
     # Ville Bergholm 2006-2014
 
@@ -527,7 +524,7 @@ def nmr_sequences(seqs=None, titles=None):
 
     if seqs == None:
         seqs = [seq.nmr([[pi, 0]]), seq.corpse(pi), seq.scrofulous(pi), seq.bb1(pi)]
-        titles = ['Plain $\pi$ pulse', 'CORPSE', 'SCROFULOUS', 'BB1']
+        titles = [r'Plain $\pi$ pulse', 'CORPSE', 'SCROFULOUS', 'BB1']
     elif titles == None:
         titles = ['User-given seq'] * len(seqs)
 
@@ -539,7 +536,7 @@ def nmr_sequences(seqs=None, titles=None):
 
     def helper(ax, s_error, title):
         """Apply the sequence on the state psi, plot the evolution."""
-        out, t = seq.propagate(psi, s_error, out_func = state.bloch_vector)
+        out, _ = seq.propagate(psi, s_error, out_func = state.bloch_vector)
         ax.set_title(title)
         plot.state_trajectory(out, ax = ax)
 
@@ -604,9 +601,7 @@ def phase_estimation(t, U, s, implicit=False):
 
        t \ge n + \left\lceil \log_2\left(2 +\frac{1}{2 \epsilon}\right) \right \rceil.
 
-    See [Cleve]_, [NC]_ chapter 5.2.
-
-    .. [Cleve] R.Cleve et al., "Quantum Algorithms Revisited", Proc. R. Soc. London A454, 339 (1998).
+    See :cite:`Cleve`, :cite:`NC` chapter 5.2.
     """
     # Ville Bergholm 2009-2010
 
@@ -633,7 +628,7 @@ def phase_estimation(t, U, s, implicit=False):
 
     if implicit:
         # save memory and CPU: make an implicit measurement of the state reg, discard the results
-        dummy, res, reg = reg.measure(t, do = 'D')
+        _, _, reg = reg.measure(t, do = 'D')
         #print('Implicit measurement of state register: {0}\n', res)
     else:
         # more expensive computationally: trace over the state register
@@ -674,7 +669,7 @@ def phase_estimation_precision(t, U, u=None):
     plt.figure()
     plt.hold(True)
     plt.bar(x, p, width = w) # TODO align = 'center' ???
-    plt.xlabel('phase / $2\pi$')
+    plt.xlabel(r'phase / $2\pi$')
     plt.ylabel('probability')
     plt.title('Phase estimation')
     #axis([-1/(T*2), 1-1/(T*2), 0, 1])
@@ -710,7 +705,7 @@ def qft_circuit(dim=(2, 3, 3, 2)):
     print('Subsystem dimensions: {0}'.format(dim))
 
     def Rgate(d):
-        """R = \sum_{xy} exp(i*2*pi * x*y/prod(dim)) |xy><xy|"""
+        r"""R = \sum_{xy} exp(i*2*pi * x*y/prod(dim)) |xy><xy|"""
         temp = kron(arange(d[0]), arange(d[-1])) / prod(d)
         return gate.phase(2 * pi * temp, (d[0], d[-1]))
 
@@ -743,6 +738,7 @@ def quantum_channels(p=0.3):
 
     print('\n\n=== Quantum channels ===\n')
 
+    I = eye(2)
     E_bitflip      = [sqrt(1-p)*I, sqrt(p)*sx]
     E_phaseflip    = [sqrt(1-p)*I, sqrt(p)*sz]
     E_bitphaseflip = [sqrt(1-p)*I, sqrt(p)*sy]
@@ -763,7 +759,7 @@ def quantum_channels(p=0.3):
 
         for a in range(s[1]):
             for b in range(s[2]):
-                temp = state.bloch_state(r_[1, S[:,a,b]])
+                temp = state.bloch_state(r_[1, S[:, a, b]])
                 res[:, a, b] = temp.kraus_propagate(E).bloch_vector()[1:]  # skip the normalization
 
         plot.bloch_sphere(ax)
@@ -794,6 +790,7 @@ def quantum_walk(steps=7, n=11, p=0.05, n_coin=2):
     p == 0 corresponds to the "fully quantum" case.
     """
     # Ville Bergholm 2010-2011
+    from .base import H
 
     print('\n\n=== Quantum walk ===\n')
     # initial state: coin shows heads, walker in center node
@@ -849,18 +846,16 @@ def qubit_and_resonator(d_r=30):
     """Qubit coupled to a microwave resonator demo.
 
     Simulates a qubit coupled to a microwave resonator.
-    Reproduces plots from the experiment in [Hofheinz]_.
-
-    .. [Hofheinz] M.Hofheinz et al., "Synthesizing arbitrary quantum states in a superconducting resonator", Nature 459, 546-549 (2009), doi:10.1038/nature08005
+    Reproduces plots from the experiment in :cite:`Hofheinz`.
     """
-    # Ville Bergholm 2010
+    # Ville Bergholm 2010-2014
 
     print('\n\n=== Qubit coupled to a single-mode microwave resonator ===\n')
     if d_r < 10:
         d_r = 10 # truncated resonator dim
 
-    omega0 = 1e9 # Energy scale/\hbar, Hz
-    T = 0.025 # K
+    #omega0 = 1e9 # Energy scale/\hbar, Hz
+    #T = 0.025 # K
 
     # omega_r = 2*pi* 6.570 # GHz, resonator angular frequency
     # qubit-resonator detuning Delta(t) = omega_q(t) -omega_r
@@ -871,11 +866,10 @@ def qubit_and_resonator(d_r=30):
     #  Omega << |Delta_off| < Omega_q << omega_r
 
     # decoherence times
-    T1_q = 650e-9 # s
-    T2_q = 150e-9 # s
-
-    T1_r = 3.5e-6 # s
-    T2_r = 2*T1_r # s
+    #T1_q = 650e-9 # s
+    #T2_q = 150e-9 # s
+    #T1_r = 3.5e-6 # s
+    #T2_r = 2*T1_r # s
 
     # TODO heat bath and couplings
     #bq = markov.bath('ohmic', omega0, T)
@@ -1053,9 +1047,7 @@ def shor_factorization(N=9, cheat=False):
     instead of simulating the quantum part (implemented in :func:`find_order`)
     we use a more efficient classical algorithm for the order-finding).
 
-    See [Shor]_, [NC]_ chapter 5.3.
-
-    .. [Shor] P.W. Shor, "Algorithms For Quantum Computation: Discrete Logs and Factoring", Proc. 35th Symp. on the Foundations of Comp. Sci., 124 (1994).
+    See :cite:`Shor`, :cite:`NC` chapter 5.3.
     """
     # Ville Bergholm 2010-2011
 
@@ -1154,7 +1146,7 @@ def find_order(a, N, epsilon=0.25):
 
     Uses :func:`phase_estimation`.
 
-    See [Shor]_, [NC]_ chapter 5.3.
+    See :cite:`Shor`, :cite:`NC` chapter 5.3.
     """
     # number of bits needed to represent mod N arithmetic:
     m = int(ceil(log2(N)))
@@ -1177,7 +1169,7 @@ def find_order(a, N, epsilon=0.25):
     dummy, num = reg.measure()
 
     def find_denominator(x, y, max_den):
-        """
+        r"""
         Finds the denominator s for r/s \approx x/y such that s < max_den
         using the convergents of the continued fraction representation
 
@@ -1253,7 +1245,7 @@ def superdense_coding(d=2):
     reg = reg.u_propagate(U.ctranspose())
     print(reg)
 
-    p, b = reg.measure()
+    _, b = reg.measure()
     b = array(np.unravel_index(b, dim))
     print('\nand measures both qudits in the computational basis, obtaining the result  b = {0}.'.format(b))
 
@@ -1299,7 +1291,7 @@ def teleportation(d=2):
     reg = reg.u_propagate(tensor(U.ctranspose(), I))
     print(reg)
 
-    p, b, reg = reg.measure((0, 1), do = 'C')
+    _, b, reg = reg.measure((0, 1), do = 'C')
     b = array(np.unravel_index(b, dim))
     print('\nand measures her qudits, getting the result {0}.'.format(b))
     print('She then transmits the two d-its to Bob using a classical channel.')
@@ -1380,6 +1372,6 @@ def tour():
     qubit_and_resonator(20)
     pause()
 
-    U = qft_circuit((2,3,2))
+    qft_circuit((2, 3, 2))
 
     #quantum_walk()
