@@ -19,34 +19,39 @@ Contents
    id
    phase
    qft
+   swap
+   walsh
    mod_inc
    mod_mul
-   swap
    mod_add
    controlled
    single
    two
-   walsh
 """
 
 from __future__ import division, absolute_import, print_function, unicode_literals
 
-from copy import deepcopy
-
-from numpy import pi, prod, diag, eye, empty, zeros, trace, exp, sqrt, mod, isscalar, kron, array, ones, array_equal
+from numpy import pi, prod, empty, zeros, trace, exp, sqrt, mod, isscalar, kron, array, ones, array_equal
 import scipy.sparse as sparse
 
-from .lmap import *
+from .lmap import lmap, tensor
 from .utils import qubits, op_list, assert_o, copy_memoize, gcd
 
 
-__all__ = ['dist', 'id', 'mod_add', 'mod_inc', 'mod_mul', 'phase', 'qft', 'swap', 'walsh', 'controlled', 'single', 'two', 'test']
+__all__ = ['dist', 'id', 'phase', 'qft', 'swap', 'walsh',
+           'mod_add', 'mod_inc', 'mod_mul', 
+           'controlled', 'single', 'two']
 
 # TODO reshape will cause problems for sparse matrices!
 # TODO utils.op_list too!
 # TODO which one is faster in element assignment -style init, dok or lil?
 # TODO make input interface consistent, do we want arrays or lmaps?
 
+def eye(D):
+    """FIXME Temp. wrapper, unnecessary after when we update to scipy 0.12
+    Then we can just do   from scipy.sparse import eye
+    """
+    return sparse.eye(D, D) 
 
 
 def dist(A, B):
@@ -75,8 +80,9 @@ def id(dim):
     Returns the identity gate I for the specified system.
     dim is a tuple of subsystem dimensions.
     """
-    if isscalar(dim): dim = (dim,)  # scalar into a tuple
-    return lmap(sparse.identity(prod(dim)), (dim, dim))
+    if isscalar(dim):
+        dim = (dim,)  # scalar into a tuple
+    return lmap(eye(prod(dim)), (dim, dim))
 
 
 def mod_add(dim1, dim2, N=None):
@@ -137,7 +143,8 @@ def mod_inc(x, dim, N=None):
     """
     # Ville Bergholm 2010
 
-    if isscalar(dim): dim = (dim,)  # scalar into a tuple
+    if isscalar(dim):
+        dim = (dim,)  # scalar into a tuple
     d = prod(dim)
     if N == None:
         N = d
@@ -169,7 +176,8 @@ def mod_mul(x, dim, N=None):
     """
     # Ville Bergholm 2010-2011
 
-    if isscalar(dim): dim = (dim,)  # scalar into a tuple
+    if isscalar(dim):
+        dim = (dim,)  # scalar into a tuple
     d = prod(dim)
     if N == None:
         N = d
@@ -199,7 +207,8 @@ def phase(theta, dim=None):
     """
     # Ville Bergholm 2011
 
-    if isscalar(dim): dim = (dim,)  # scalar into a tuple
+    if isscalar(dim):
+        dim = (dim,)  # scalar into a tuple
     n = len(theta)
     if dim == None:
         dim = (n,)
@@ -207,7 +216,7 @@ def phase(theta, dim=None):
     if d != n:
         raise ValueError('Dimension mismatch.')
 
-    return lmap(sparse.spdiags(exp(1j * theta), 0, d, d, 'csr') , (dim, dim))
+    return lmap(sparse.diags(exp(1j * theta), 0) , (dim, dim))
 
 
 @copy_memoize
@@ -220,8 +229,8 @@ def qft(dim):
     """
     # Ville Bergholm 2004-2011
 
-    if isscalar(dim): dim = (dim,)  # scalar into a tuple
-    n = len(dim)
+    if isscalar(dim):
+        dim = (dim,)  # scalar into a tuple
     N = prod(dim)
     U = empty((N, N), complex)  # completely dense, so we don't have to initialize it with zeros
     for j in range(N):
@@ -231,7 +240,7 @@ def qft(dim):
 
 
 def swap(d1, d2):
-    """Swap gate.
+    r"""Swap gate.
 
     Returns the swap gate which swaps the order of two subsystems with dimensions [d1, d2].
 
@@ -261,7 +270,7 @@ def walsh(n):
     from .base import H
 
     U = 1
-    for k in range(n):
+    for _ in range(n):
         U = kron(U, H)
     dim = qubits(n)
     return lmap(U, (dim, dim))
@@ -280,7 +289,7 @@ def controlled(U, ctrl=(1,), dim=None):
     dim is the dimensions vector for the control qudits. If not given, all controls
     are assumed to be qubits.
 
-    Examples:
+    Examples::
 
       controlled(NOT, [1]) gives the standard CNOT gate.
       controlled(NOT, [1, 1]) gives the Toffoli gate.
@@ -288,7 +297,8 @@ def controlled(U, ctrl=(1,), dim=None):
     # Ville Bergholm 2009-2011
 
     # TODO generalization, uniformly controlled gates?
-    if isscalar(dim): dim = (dim,)  # scalar into a tuple
+    if isscalar(dim):
+        dim = (dim,)  # scalar into a tuple
     t = len(ctrl)
     if dim == None:
         dim = qubits(t) # qubits by default
@@ -322,9 +332,7 @@ def controlled(U, ctrl=(1,), dim=None):
 
     # controlled gates only make sense for square matrices U (we need an identity transformation for the 'no' cases!)
     U_dim = U.shape[0]
-    S = U_dim * T
-
-    out = sparse.spdiags(kron(no, ones(U_dim)), 0, S, S) + sparse.kron(sparse.spdiags(yes, 0, T, T), U)
+    out = sparse.diags(kron(no, ones(U_dim)), 0) +sparse.kron(sparse.diags(yes, 0), U)
     return lmap(out, (d1, d2))
 
 
@@ -385,38 +393,10 @@ def two(B, t, d_in):
         p = [0, 2, 1]
     else:
         p = [1, 2, 0]
-    U = tensor(B, lmap(sparse.identity(inbetween))).reorder((p, p), inplace = True)
-    U = tensor(lmap(sparse.identity(before)), U, lmap(sparse.identity(after)))
+    U = tensor(B, lmap(eye(inbetween))).reorder((p, p), inplace = True)
+    U = tensor(lmap(eye(before)), U, lmap(eye(after)))
 
     # restore dimensions
     d_out = d_in.copy()
     d_out[t] = B.dim[0]
     return lmap(U, (d_out, d_in))
-
-
-def test():
-    """Test script for gates.
-    """
-    # Ville Bergholm 2010
-
-    from numpy.random import randn
-    from .base import sx, sy, sz, tol
-
-    dim = (2, 4)
-
-    I = id(dim)
-    U = swap(*dim)
-    assert_o((U.ctranspose() * U - I).norm(), 0, tol)  # swap' * swap = I
-
-    # TODO test the output
-    U = mod_add(2, 4, 3)
-    U = mod_inc(3, dim, 5)
-    V = mod_mul(2, dim, 5)
-    dist(U, V)
-    U = phase(randn(prod(dim)), dim)
-    U = qft(dim)
-    U = walsh(3)
-    U = controlled(sz, (1, 0), dim)
-    cnot = controlled(sx)
-    U = single(sy, 0, dim)
-    U = two(cnot, (2,0), (2,3,2))
