@@ -25,17 +25,18 @@ Contents
 from __future__ import division, absolute_import, print_function, unicode_literals
 
 import numpy as np
-from numpy import array, asarray, arange, empty, zeros, ones, sqrt, sin, cos, dot, sort, trace, kron, pi, r_, c_, linspace, meshgrid, roll, concatenate, angle
+from numpy import (array, asarray, arange, empty, zeros, ones, sqrt, sin, cos, dot, sort, trace, kron,
+    pi, r_, c_, linspace, meshgrid, roll, concatenate, angle)
 from numpy.linalg import det, eigvals
 from scipy.linalg import norm
 import matplotlib.pyplot as plt
-import mpl_toolkits.mplot3d
 
-from .base import *
-from .lmap import *
+from .base import sy, Q_Bell
+from .lmap import lmap, tensor
+
 
 # TODO these function names are terrible
-__all__ = ['canonical', 'makhlin', 'max_concurrence', 'plot_weyl_2q', 'plot_makhlin_2q', 'LU', 'test']
+__all__ = ['canonical', 'makhlin', 'max_concurrence', 'plot_weyl_2q', 'plot_makhlin_2q', 'LU']
 
 
 def LU(rho, k, perms):
@@ -49,17 +50,15 @@ def LU(rho, k, perms):
     This function can be very inefficient for some invariants, since
     it does no partial traces etc. which might simplify the calculation.
 
-    Uses the algorithm in [BBL2012]_.
-
-    .. [BBL2012] J.D.Biamonte, V.Bergholm, M.Lanzagorta, "Tensor network methods for invariant theory", J.Phys.A 46, 475301 (2013). doi:10.1088/1751-8113/46/47/475301
+    Uses the algorithm in :cite:`BBL2012`.
     """
     # Ville Bergholm 2011-2012
 
     def tensor_pow(rho, n):
         """Returns $\rho^{\otimes n}$."""
-        rho = rho.to_op()
+        rho.to_op(inplace=True)
         ret = lmap(rho)
-        for k in range(1, n):
+        for _ in range(1, n):
             ret = tensor(ret, rho)
         return ret
 
@@ -69,10 +68,11 @@ def LU(rho, k, perms):
         raise ValueError('Need one permutation per subsystem.')
 
     # convert () to identity permutation
+    id_perm = tuple(range(k))
     perms = list(perms)
     for j, p in enumerate(perms):
         if len(p) == 0:
-            perms[j] = range(k)
+            perms[j] = id_perm
 
     # splice k sequential copies of the entire system into k copies of each subsystem
     s = arange(n * k).reshape((k, n)).flatten()
@@ -96,9 +96,7 @@ def canonical(U):
 
     Returns a vector of three real canonical local invariants for the
     U(4) matrix U, normalized to the range [0,1].
-    Uses the algorithm in [Childs]_.
-
-    .. [Childs] Childs et al., "Lower bounds on the complexity of simulating quantum gates", PRA 68, 052311 (2003).
+    Uses the algorithm in :cite:`Childs`.
     """
     # Ville Bergholm 2004-2010
 
@@ -134,15 +132,11 @@ def canonical(U):
 def makhlin(U):
     """Makhlin local invariants of a two-qubit gate.
 
-    Returns a vector of the three real Makhlin invariants (see [Makhlin]_) corresponding
+    Returns a vector of the three real Makhlin invariants (see :cite:`Makhlin`) corresponding
     to the U(4) gate U.
 
-    Alternatively, given a vector of canonical invariants
-    normalized to [0, 1], returns the corresponding Makhlin invariants
-    (see [Zhang]_).
-
-    .. [Makhlin] Yu. Makhlin, "Nonlocal Properties of Two-Qubit Gates and Mixed States, and the Optimization of Quantum Computations", QIP 1, 243 (2002).
-    .. [Zhang] J.Zhang et al., "Geometric theory of nonlocal two-qubit operations", PRA 67, 042313 (2003).
+    Alternatively, given a vector of canonical invariants normalized to [0, 1],
+    returns the corresponding Makhlin invariants (see :cite:`Zhang`).
     """
     # Ville Bergholm 2004-2010
 
@@ -152,9 +146,9 @@ def makhlin(U):
         c *= pi
         g = empty(c.shape)
         
-        g[...,0] = (cos(c[...,0]) * cos(c[...,1]) * cos(c[...,2])) ** 2 - (sin(c[...,0]) * sin(c[...,1]) * sin(c[...,2])) ** 2
-        g[...,1] = 0.25 * sin(2 * c[...,0]) * sin(2 * c[...,1]) * sin(2 * c[...,2])
-        g[...,2] = 4 * g[...,0] - cos(2 * c[...,0]) * cos(2 * c[...,1]) * cos(2*c[...,2])
+        g[..., 0] = (cos(c[..., 0]) * cos(c[..., 1]) * cos(c[..., 2])) ** 2 -(sin(c[..., 0]) * sin(c[..., 1]) * sin(c[..., 2])) ** 2
+        g[..., 1] = 0.25 * sin(2 * c[..., 0]) * sin(2 * c[..., 1]) * sin(2 * c[..., 2])
+        g[..., 2] = 4 * g[..., 0] - cos(2 * c[..., 0]) * cos(2 * c[..., 1]) * cos(2*c[..., 2])
     else:
         # U(4) gate matrix    
         V = dot(Q_Bell.conj().transpose(), dot(U, Q_Bell))
@@ -170,12 +164,10 @@ def max_concurrence(U):
     """Maximum concurrence generated by a two-qubit gate.
 
     Returns the maximum concurrence generated by the two-qubit
-    gate U (see [Kraus]_), starting from a tensor state.
+    gate U (see :cite:`Kraus`), starting from a tensor state.
 
     Alternatively, U may be given in terms of a vector of three
     canonical local invariants.
-
-    .. [Kraus] B.Kraus and J. I. Cirac, "Optimal creation of entanglement using a two-qubit gate", PRA 63, 062309 (2001).
     """
     # Ville Bergholm 2006-2010
 
@@ -221,7 +213,8 @@ def plot_makhlin_2q(sdiv=31, tdiv=31):
     ax = fig.add_subplot(111, projection='3d')
 
     # mesh, waterfall?
-    polyc = ax.plot_surface(G[:,:,0], G[:,:,1], G[:,:,2], rstride = 1, cstride = 1, cmap = cm.jet, norm = colors.Normalize(vmin=0, vmax=1, clip=True), alpha = 0.6)
+    polyc = ax.plot_surface(G[:, :, 0], G[:, :, 1], G[:, :, 2], rstride = 1, cstride = 1,
+        cmap = cm.jet, norm = colors.Normalize(vmin=0, vmax=1, clip=True), alpha = 0.6)
     polyc.set_array(C.ravel() ** 2)  # FIXME colors
     ax.axis('equal')
     #ax.axis([-1, 1, -0.5, 0.5, -3, 3])
@@ -240,7 +233,7 @@ def plot_makhlin_2q(sdiv=31, tdiv=31):
     ax.text(0.1, 0.26, 0, 'SWAP$^{1/2}$')
     ax.text(0, -0.26, 0, 'SWAP$^{-1/2}$')
 
-    cb = fig.colorbar(polyc, ax = ax)
+    fig.colorbar(polyc, ax = ax)
     plt.show()
     return ax
 
@@ -249,7 +242,7 @@ def plot_weyl_2q(ax=None):
     """Plots the two-qubit Weyl chamber.
 
     Plots the Weyl chamber for the local invariants
-    of 2q gates. See [Zhang]_.
+    of 2q gates. See :cite:`Zhang`.
 
     Returns the Axes3D object.
     """
@@ -276,30 +269,3 @@ def plot_weyl_2q(ax=None):
     ax.text(0.20, 0.25, 0, 'SWAP$^{1/2}$')
     ax.text(0.75, 0.25, 0, 'SWAP$^{-1/2}$')
     return ax
-
-
-
-def test():
-    """Test script for invariant methods."""
-    from .utils import assert_o, rand_U
-    from . import gate
-
-    U = rand_U(4) # random two-qubit gate
-    L = kron(rand_U(2), rand_U(2)) # random local 2-qubit gate
-    cnot = gate.controlled(sx).data
-    swap = gate.swap(2,2).data
-
-    # canonical invariants
-    #assert_o(norm(canonical(L) -[0, 0, 0]), 0, tol) # only point in Berkeley chamber with translation degeneracy, (0,0,0) =^ (1,0,0)
-    assert_o(norm(canonical(cnot) -[0.5, 0, 0]), 0, tol)
-    assert_o(norm(canonical(swap) -[0.5, 0.5, 0.5]), 0, tol)
-
-    # Makhlin invariants
-    c = canonical(U)
-    g1 = makhlin(c)
-    g2 = makhlin(U)
-    assert_o(norm(g1-g2), 0, tol)
-
-    # maximum concurrence
-    assert_o(max_concurrence(L), 0, tol)
-    assert_o(max_concurrence(cnot), 1, tol)
