@@ -47,7 +47,6 @@ Non-member functions:
 
 from __future__ import division, absolute_import, print_function, unicode_literals
 
-import sys
 from copy import copy, deepcopy
 
 import numpy as np
@@ -61,12 +60,12 @@ __all__ = ['numstr_to_array', 'array_to_numstr', 'lmap', 'tensor']
 
 def numstr_to_array(s):
     """Utility, converts a numeric string to the corresponding array."""
-    return np.array(map(lambda x: ord(x) - ord('0'), s))
+    return np.array([ord(x) -ord('0') for x in s])
 
 
 def array_to_numstr(s):
     """Utility, converts an integer array to the corresponding numeric string."""
-    return "".join(map(lambda x: chr(x + ord('0')), s))
+    return "".join([chr(x +ord('0')) for x in s])
 
 
 
@@ -239,11 +238,11 @@ class lmap(object):
         NOTE: changes the object itself!
         """
         dd = []
-        for d in self.dim[:]:
-          temp = filter(lambda x: x > 1, d)
-          if len(temp) == 0:
-              temp = (1,)
-          dd.append(temp)
+        for d in self.dim:
+            temp = tuple([x for x in d if x > 1])
+            if len(temp) == 0:
+                temp = (1,)
+            dd.append(temp)
         self.dim = tuple(dd)
         return
 
@@ -425,7 +424,7 @@ class lmap(object):
         for k, this_perm in enumerate(perm):
             # avoid a subtle problem with the input syntax, (0, 1) must not be understood as swap!
             if this_perm != None and tuple(this_perm) == (0, 1):
-                this_perm = None;
+                this_perm = None
 
             # requested permutation for this index
             if this_perm == None:
@@ -464,45 +463,6 @@ class lmap(object):
         return s
 
 
-    @staticmethod
-    def test():
-        """Test script for the lmap module.
-        """
-        # Ville Bergholm 2009-2011
-        from numpy.testing import assert_almost_equal
-        from numpy.random import rand, randn
-
-        decimal = -int(np.log10(tol))
-
-        # reordering subsystems
-        idim = (2, 5, 3)
-        odim = (4, 3, 2)
-        A = lmap(rand(odim[0], idim[0]))
-        B = lmap(rand(odim[1], idim[1]))
-        C = lmap(rand(odim[2], idim[2]))
-        T1 = tensor(A, B, C)
-
-        p = (2, 0, 1)
-        T2 = T1.reorder((p, p))
-        assert_almost_equal((tensor(C, A, B) - T2).norm(), 0, decimal)
-
-        p = (1, 0, 2)
-        T2 = T1.reorder((p, p))
-        assert_almost_equal((tensor(B, A, C) - T2).norm(), 0, decimal)
-
-        p = (2, 1, 0)
-        T2 = T1.reorder((p, p))
-        assert_almost_equal((tensor(C, B, A) - T2).norm(), 0, decimal)
-
-        ignore = """
-        a = lmap(randn(2,4) +1j*randn(2,4), (None, (2,2)))
-        b = lmap(randn(2,2) +1j*randn(2,2))
-        c = lmap(randn(2))
-
-        print(repr(b * c))
-        print(repr(tensor(a, b)))
-        """
-
 
 def tensor(*arg):
     """Tensor product of lmaps."""
