@@ -7,9 +7,7 @@ Born-Markov noise (:mod:`qit.markov`)
 This module simulates the effects of a heat bath coupled to a quantum
 system, using the Born-Markov approximation.
 
-The treatment in this module mostly follows [BP]_.
-
-.. [BP] Breuer, Heinz-Peter and Petruccione, Francesco, "The Theory of Open Quantum Systems", Oxford University Press (2002).
+The treatment in this module mostly follows Ref. :cite:`BP`.
 
 
 
@@ -39,20 +37,21 @@ Contents
    corr
    fit
 """
-# Ville Bergholm 2011
+# Ville Bergholm 2011-2014
 
 from __future__ import division, absolute_import, print_function, unicode_literals
 
-from numpy import array, sqrt, exp, sin, cos, arctan2, tanh, dot, argsort, pi, r_, linspace, logspace, searchsorted, inf, newaxis, isscalar, unravel_index
+from numpy import (array, sqrt, exp, sin, cos, arctan2, tanh, dot, argsort, pi,
+    r_, linspace, logspace, searchsorted, inf, newaxis, unravel_index)
 from scipy.linalg import norm
 from scipy.integrate import quad
 import scipy.constants as const
 
-from .base import *
-from .utils import *
+from .base import sx, sz, tol
+from .utils import lmul, rmul, lrmul, rand_hermitian, superop_lindblad, spectral_decomposition
 
 
-__all__ = ['bath', 'ops', 'lindblad_ops', 'test']
+__all__ = ['bath', 'ops', 'lindblad_ops', 'superop']
 
 
 class bath(object):
@@ -213,7 +212,7 @@ class bath(object):
         # clear lookup tables, since changing the cutoff requires recalc of S
         # start with a single point precomputed
         self.dH = array([-inf, 0, inf])
-        self.gs_table = array([[0, 0], [self.g0, self.s0], [0,0]])
+        self.gs_table = array([[0, 0], [self.g0, self.s0], [0, 0]])
 
 
 
@@ -250,6 +249,7 @@ class bath(object):
         d2 = abs(x - ee[1])
 
         def interpolate(ee, tt, x):
+            "Quick interpolation."
             # interp1 does way too many checks
             return tt[0] + ((x - ee[0]) / (ee[1] - ee[0])) * (tt[1] - tt[0])
 
@@ -315,7 +315,7 @@ class bath(object):
 
             iTd = 1 / T2 -0.5 / T1 # inverse pure dephasing time
             if iTd < 0:
-                error('Unphysical decoherence times!')
+                raise ValueError('Unphysical decoherence times!')
     
             # match bath couplings to T1, T2
             temp = self.scale * delta / 2
@@ -510,31 +510,3 @@ def superop(H, D, B):
             diss  += lrmul(g * A[k].conj().transpose(), A[k]) # here too
 
     return lmul(acomm -iH_LS) +rmul(acomm +iH_LS) +diss
-
-
-
-def test():
-    """Test script for Born-Markov methods.
-    """
-    # Ville Bergholm 2009-2010
-
-    dim = 6
-
-    H = rand_hermitian(dim)
-    D = [rand_hermitian(dim)/10, rand_hermitian(dim)/10]
-    B = [bath('ohmic', 1e9, 0.02), bath('ohmic', 1e9, 0.03)]
-
-    # jump operators
-    dH, X = ops(H, D)
-    assert_o(dH[0], 0, tol)
-    for n, A in enumerate(X):
-        temp = A[0] # dH[0] == 0
-        for k in range(1, len(dH)):
-            temp += A[k] +A[k].conj().transpose() # A(-omega) == A'(omega)
-        assert_o(norm(temp - D[n]), 0, tol) # Lindblad ops should sum to D
-
-    # equivalence of Lindblad operators and the Liouvillian superoperator
-    LL, H_LS = lindblad_ops(H, D, B)
-    S1 = superop_lindblad(LL, H + H_LS)
-    S2 = superop(H, D, B)
-    assert_o(norm(S1 - S2), 0, tol)
