@@ -1,5 +1,5 @@
 # -*- coding: utf-8 -*-
-"""
+r"""
 Control sequences (:mod:`qit.seq`)
 ==================================
 
@@ -7,26 +7,21 @@ Control sequences (:mod:`qit.seq`)
 Piecewise constant control sequences for quantum systems.
 Each control sequence is a dictionary with the following keys:
 
-  A
-    Drift generator (typically :math:`i/\hbar` times a Hamiltonian and
-    a time unit of your choice).
-
-  B
-    List of control generators. c := len(B).
-
-  tau
-    Vector of durations of the time slices. m := len(tau).
-
-  control
-    Array, shape == (m, c). control[i,j] is the value of control
-    field j during time slice i.
+=======  ===============================================================================================
+A        Drift generator (typically :math:`i/\hbar` times a Hamiltonian and a time unit of your choice).
+B        List of control generators. c := len(B).
+tau      Vector of durations of the time slices. m := len(tau).
+control  Array, shape (m, c). control[i,j] is the value of control field j during time slice i.
+=======  ===============================================================================================
 
 The total generator for the time slice j is thus given by
+
 .. math::
 
    G_j = A +\sum_k \text{control}_{jk} B_k,
 
 and the corresponding propagator is
+
 .. math::
 
    P_j = \exp(-\tau_j G_j).
@@ -50,7 +45,7 @@ Contents
    seq2prop
    propagate
 """
-# Ville Bergholm 2011
+# Ville Bergholm 2011-2014
 
 from __future__ import division, absolute_import, print_function, unicode_literals
 
@@ -58,10 +53,10 @@ from numpy import array, sin, cos, arcsin, arccos, pi, asarray, eye, zeros, r_,
 from scipy.linalg import expm
 from scipy.optimize import brentq
 
-from .base import *
+from .base import sx, sy, tol
 
 
-__all__ = ['nmr', 'bb1', 'corpse', 'cpmg', 'scrofulous', 'seq2prop', 'propagate', 'test']
+__all__ = ['nmr', 'bb1', 'corpse', 'cpmg', 'scrofulous', 'seq2prop', 'propagate']
 
 
 def nmr(a):
@@ -85,7 +80,7 @@ def nmr(a):
     """
     # Ville Bergholm 2006-2009
 
-    a = asarray(a)
+    a = asarray(a, dtype=float)
     theta = a[:, 0]
     phi   = a[:, 1]
 
@@ -108,11 +103,9 @@ def bb1(theta, phi=0, location=0.5):
     r"""Sequence for correcting pulse length errors.
 
     Returns the Broadband number 1 control sequence for correcting
-    proportional errors in pulse length (or amplitude) [Wimperis]_.
+    proportional errors in pulse length (or amplitude) :cite:`Wimperis`.
 
     The target rotation is :math:`\theta_\phi` in the NMR notation.
-
-    .. [Wimperis] S.Wimperis, "Broadband, Narrowband, and Passband Composite Pulses for Use in Advanced NMR Experiments", J. Magn. Reson. A 109, 221--231 (1994).
     """
     # Ville Bergholm 2009-2012
 
@@ -126,13 +119,11 @@ def corpse(theta, phi=0):
 
     Returns the CORPSE control sequence for correcting off-resonance
     errors, i.e. ones arising from a constant but unknown
-    :math:`\sigma_z` bias in the Hamiltonian [Cummins]_
+    :math:`\sigma_z` bias in the Hamiltonian :cite:`Cummins`
 
     The target rotation is :math:`\theta_\phi` in the NMR notation.
 
     CORPSE: Compensation for Off-Resonance with a Pulse SEquence
-
-    .. [Cummins] Cummins et al., "Tackling systematic errors in quantum logic gates with composite rotations", PRA 67, 042308 (2003).
     """
     # Ville Bergholm 2009
 
@@ -172,20 +163,20 @@ def scrofulous(theta, phi=0):
     r"""Sequence for correcting pulse length errors.
 
     Returns the SCROFULOUS control sequence for correcting errors
-    in pulse duration (or amplitude) [Cummins]_.
+    in pulse duration (or amplitude) :cite:`Cummins`.
 
     The target rotation is :math:`\theta_\phi` in the NMR notation.
 
     SCROFULOUS: Short Composite ROtation For Undoing Length Over- and UnderShoot
     """
-    # Ville Bergholm 2006-2012
+    # Ville Bergholm 2006-2014
 
     th1 = brentq(lambda t: (sin(t)/t -(2 / pi) * cos(theta / 2)), 0.1, 4.6)
     ph1 = arccos(-pi * cos(th1) / (2 * th1 * sin(theta / 2)))
     ph2 = ph1 - arccos(-pi / (2 * th1))
 
-    u1 = [[th1, ph1]]
-    u2 = [[pi,  ph2]]
+    u1 = [[th1, ph1 +phi]]
+    u2 = [[pi,  ph2 +phi]]
     return nmr(u1 + u2 + u1)
 
 
@@ -199,8 +190,8 @@ def seq2prop(s):
     """
     # Ville Bergholm 2009-2012
 
-    A = s['A'];
-    B = s['B'];
+    A = s['A']
+    B = s['B']
 
     n = len(s['tau'])
     P = eye(A.shape[0])
@@ -222,8 +213,8 @@ def propagate(s, seq, out_func=lambda x: x, base_dt=0.1):
     """
     # Ville Bergholm 2009-2014
 
-    A = seq['A'];
-    B = seq['B'];
+    A = seq['A']
+    B = seq['B']
 
     n = len(seq['tau'])
     t = [0]  # initial time
@@ -247,24 +238,3 @@ def propagate(s, seq, out_func=lambda x: x, base_dt=0.1):
         temp = t[-1]
         t.extend(list(linspace(temp+dt, temp+T, n_steps)))
     return out, t
-
-
-
-def test():
-    """Test script for the control sequences module.
-    """
-    # Ville Bergholm 2011
-
-    from numpy.random import rand
-    from . import state
-    from .utils import rand_positive, assert_o
-
-    dim = 2
-    s = state(rand_positive(dim))
-    seq = scrofulous(pi*rand(), 2*pi*rand())
-
-    # equivalent propagations
-    s1 = s.u_propagate(seq2prop(seq))
-    out, t = propagate(s, seq)
-    s2 = out[-1]
-    assert_o((s1-s2).norm(), 0, tol)
