@@ -72,9 +72,6 @@ Other state representations
 
    bloch_vector
    bloch_state
-
-
-.. [NC] M.A. Nielsen, I.L. Chuang, "Quantum Computation and Quantum Information" (2000).
 """
 # Ville Bergholm 2008-2012
 
@@ -86,16 +83,19 @@ import types
 from copy import deepcopy
 
 import numpy as np
-from numpy import array, asarray, empty, diag, sort, prod, cumsum, cumprod, exp, sqrt, trace, dot, vdot, roll, zeros, ones, r_, kron, isscalar, nonzero, ix_, linspace, meshgrid
+from numpy import (array, asarray, empty, diag, sort, prod, cumsum, cumprod, exp, sqrt, trace,
+    dot, vdot, roll, zeros, ones, r_, kron, isscalar, nonzero, ix_, linspace, meshgrid)
 from numpy.random import rand, randn
 import scipy as sp
-from scipy.linalg import norm, eigh, eigvalsh
+from scipy.linalg import norm, eigh, eigvalsh, sqrtm, svd, svdvals, det
 from scipy.integrate import ode
 
-from .base import Q_Bell, tol
-from .lmap import *
-from .utils import *
-from .utils import _warn
+from .base import sy, Q_Bell, tol
+from .lmap import tensor as lmap_tensor
+from .lmap import numstr_to_array, array_to_numstr, lmap
+from .utils import (_warn, vec, inv_vec, qubits, expv, rand_U, rand_SU, rand_positive, mkron,
+    eighsort, spectral_decomposition, majorize, tensorbasis, assert_o)
+
 
 __all__ = ['equal_dims', 'index_muls', 'state', 'fidelity', 'trace_dist']
 
@@ -134,7 +134,7 @@ class state(lmap):
 
     def __init__(self, s, dim=None):
         """Construct a state.
-        
+
         calling syntax            result
         ==============            ======
         state('00101')            standard basis ket |00101> in a five-qubit system
@@ -154,6 +154,8 @@ class state(lmap):
           W,
           Bell1, Bell2, Bell3, Bell4 
         """
+        from six import string_types
+
         # we want a tuple for dim
         if isinstance(dim, collections.Iterable):
             dim = tuple(dim)
@@ -169,12 +171,11 @@ class state(lmap):
                 if s.dim[0] != s.dim[1]:
                     raise ValueError('State operator must be square.')
                 dim = (dim, dim)
-
             # call the lmap copy constructor
-            lmap.__init__(self, s, dim)
+            super(state, self).__init__(s, dim)
             return
 
-        elif isinstance(s, basestring):
+        elif isinstance(s, string_types):
             # string
 
             if s[0].isalpha():
@@ -224,7 +225,8 @@ class state(lmap):
 
             dim = (dim, (1,))  # ket
 
-        elif isinstance(s, numbers.Number):
+        elif isinstance(s, (numbers.Number, np.number)):
+            # FIXME when numpy number hierarchy inherits the Python one
             # integer defining a standard basis ket
             if dim == None:
                 raise ValueError('Need system dimension.')
@@ -254,8 +256,7 @@ class state(lmap):
 
         # now s is an ndarray
         # call the lmap constructor
-        lmap.__init__(self, s, dim)
-
+        super(state, self).__init__(s, dim)
 
 
 # utility methods
@@ -420,6 +421,7 @@ class state(lmap):
         keep = s.invert_selection(sys)
 
         def tensorsum(a, b):
+            """Like tensor product, but with sum."""
             #c = log(kron(exp(a), exp(b))) # a perverted way of doing it, the exp overflows...
             c = []
             for k in a:
@@ -672,12 +674,15 @@ class state(lmap):
             # derivative functions for the solver TODO vectorization?
             # H, ket
             def pure_fun(t, y, F):
+                "Derivative of a pure state, Hamiltonian."
                 return -1j * dot(F(t), y)
             def pure_jac(t, y, F):
+                "Jacobian of a pure state, Hamiltonian."
                 return -1j * F(t)
 
             # H, state op
             def mixed_fun(t, y, F):
+                "Derivative of a mixed state, Hamiltonian."
                 H = -1j * F(t)
                 if y.ndim == 1:
                     rho = inv_vec(y, dim)  # into a matrix
@@ -692,12 +697,15 @@ class state(lmap):
 
             # L, state op, same as the H/ket ones, only without the -1j
             def liouvillian_fun(t, y, F):
+                "Derivative of a state, Liouvillian."
                 return dot(F(t), y)
             def liouvillian_jac(t, y, F):
+                "Jacobian of a state, Liouvillian."
                 return F(t)
 
             # A, state op
             def lindblad_fun(t, y, F):
+                "Derivative of a mixed state, Lindbladian."
                 X = F(t)  # X == [H, A_1, A_2, ..., A_n]
                 H = -1j * X[0] # -1j * Hamiltonian
                 Lind = X[1:]   # Lindblad ops
@@ -768,13 +776,13 @@ class state(lmap):
                     # FIXME imaginary time doesn't yet work
                     w, err, hump = expv(-1j * t, H, s.data)
                     for k in range(n):
-                        s.data = w[k,:]  # TODO state ops
+                        s.data = w[k, :]  # TODO state ops
                         out.append(out_func(s, H))
             elif gen == 'L':
                 # Krylov subspace method
                 w, err, hump = expv(t, H, vec(s.data))
                 for k in range(n):
-                    s.data = inv_vec(w[k,:])
+                    s.data = inv_vec(w[k, :])
                     out.append(out_func(s, H))
 
         if len(out) == 1:
@@ -796,10 +804,11 @@ class state(lmap):
         # TODO: If n > prod(dims(s))^2, there is a simpler equivalent
         # operation. Should the user be notified?
         def test_kraus(E):
+            "Check if E represents a physical quantum operation."
             temp = 0
             for k in E:
                 temp += dot(k.conj().transpose(), k)
-            if norm(temp.data - eye(temp.shape)) > tol:
+            if norm(temp.data - np.eye(temp.shape)) > tol:
                 _warn('Unphysical quantum operation.')
 
         if self.is_ket():
@@ -1001,7 +1010,7 @@ class state(lmap):
         For state vectors this is equivalent to the overlap, :math:`F = |\langle a|b\rangle|`.
 
         Fidelity is symmetric in its arguments and bounded in the interval [0,1].
-        See [NC]_, chapter 9.2.2.
+        See :cite:`NC`, chapter 9.2.2.
         """
         # Ville Bergholm 2009-2010
 
@@ -1017,8 +1026,8 @@ class state(lmap):
             if r.is_ket():
                 return sqrt(vdot(r.data, dot(self.data, r.data)).real)  # .real
             else:
-                temp = sp.linalg.sqrtm(self.data)
-                return trace(sp.linalg.sqrtm(dot(dot(temp, r.data), temp))).real  # .real
+                temp = sqrtm(self.data)
+                return trace(sqrtm(dot(dot(temp, r.data), temp))).real  # .real
 
 
     def trace_dist(self, r):
@@ -1028,7 +1037,7 @@ class state(lmap):
         :math:`D(r, s) = \frac{1}{2} \mathrm{Tr}(\sqrt{A^\dagger A})`, where A = r-s.
 
         Equivalently :math:`D(r, s) = \frac{1}{2} \sum_k |\lambda_k|`, where :math:`\lambda_k`
-        are the eigenvalues of A (since A is Hermitian). See [NC]_, chapter 9.2.1.
+        are the eigenvalues of A (since A is Hermitian). See :cite:`NC`, chapter 9.2.1.
         """
         # Ville Bergholm 2009
 
@@ -1065,11 +1074,11 @@ class state(lmap):
 
         The state is then given by :math:`\sum_k \lambda_k |k\rangle_A \otimes |k\rangle_B`.
 
-        See [NC]_, chapter 2.5.
+        See :cite:`NC`, chapter 2.5.
         """
-        # Ville Bergholm 2009-2010
+        # Ville Bergholm 2009-2014
 
-        dim = self.dims()
+        dim = array(self.dims())
         n = self.subsystems()
 
         if sys == None:
@@ -1100,9 +1109,9 @@ class state(lmap):
 
         # order the coefficients into a matrix, take an svd
         if not full:
-            return sp.linalg.svdvals(s.data.reshape(d1, d2))
+            return svdvals(s.data.reshape(d1, d2))
         else:
-            u, s, vh = sp.linalg.svd(s.data.reshape(d1, d2), full_matrices = False)
+            u, s, vh = svd(s.data.reshape(d1, d2), full_matrices = False)
             # note the definition of vh in svd
             return s, u, vh.transpose()
 
@@ -1148,13 +1157,10 @@ class state(lmap):
         Returns the concurrence of the state s wrt. the partitioning
         given by the listing of subsystems in the vector sys.
 
-        See [Wootters]_, [Horodecki]_.
-
-        .. [Wootters] W.K. Wootters, "Entanglement of Formation of an Arbitrary State of Two Qubits", PRL 80, 2245 (1998).
-        .. [Horodecki] R.\ Horodecki, P. Horodecki, M. Horodecki, K. Horodecki, "Quantum entanglement", arXiv:quant-ph/0702225 (2007).
+        See :cite:`Wootters`, :cite:`Horodecki`.
         """
-        # Ville Bergholm 2006-2010
-
+        # Ville Bergholm 2006-2014
+        # TODO rewrite, check
         if abs(self.trace() - 1) > tol:
             _warn('State not properly normalized.')
 
@@ -1169,10 +1175,9 @@ class state(lmap):
                 raise ValueError('Not a pure state.')
 
             # pure state
-            n = len(dim)
+            #n = len(dim)
             rho_A = self.ptrace(self.invert_selection(sys)) # trace over everything but sys
-            C = 2 * sqrt(det(rho_A.data).real) # = sqrt(2*(1-real(trace(temp*temp)))), .real
-            return
+            return 2 * sqrt(det(rho_A.data).real) # = sqrt(2*(1-real(trace(temp*temp)))), .real
 
         # concurrence between two qubits
         if self.subsystems() != 2 or any(dim != array([2, 2])):
@@ -1183,7 +1188,7 @@ class state(lmap):
         p = self.data
         if self.is_ket():
             # ket
-            C = abs(dot(dot(p.transpose(), X), p))
+            return abs(dot(dot(p.transpose(), X), p))
 
             # find the coefficients a of the state ket in the magic base
             # phi+-, psi+-,  = triplet,singlet
@@ -1194,11 +1199,11 @@ class state(lmap):
             # state operator
             temp = dot(p, X)  # X.conj() == X so this works
             temp = dot(temp, temp.conj())  # == p * X * conj(p) * X
-            if abs(purity(self) - 1) > tol:
+            if abs(self.purity() - 1) > tol:
                 L = sqrt(sort(np.linalg.eigvals(temp).real)[::-1]).real  # .real?
-                C = max(0, L[1] -L[2] -L[3] -L[4])
+                return max(0, L[1] -L[2] -L[3] -L[4])
             else:
-                C = sqrt(trace(temp).real) # same formula as for state vecs, .real?
+                return sqrt(trace(temp).real) # same formula as for state vecs, .real?
 
 
     def negativity(self, sys):
@@ -1207,15 +1212,12 @@ class state(lmap):
         Returns the negativity of the state wrt. the partitioning
         given by the listing of subsystems in the vector sys.
 
-        See [Peres]_, [Horodecki1]_
-
-        .. [Peres] A.Peres, "Separability Criterion for Density Matrices", PRL 77, 1413 (1996).
-        .. [Horodecki1] M.Horodecki et al., "Separability of Mixed States: Necessary and Sufficient Conditions", Physics Letters A 223, 1-8 (1996).
+        See :cite:`Peres`, :cite:`Horodecki1`
         """
         # Ville Bergholm 2008-2014
 
         s = self.ptranspose(sys)  # partial transpose the state
-        x = sp.linalg.svdvals(s.data)  # singular values
+        x = svdvals(s.data)  # singular values
         return (sum(x) - 1) / 2
 
 
@@ -1225,9 +1227,9 @@ class state(lmap):
         Returns the logarithmic negativity of the state wrt. the partitioning
         given by the listing of subsystems in the vector sys.
         """
-        # Ville Bergholm 2008
+        # Ville Bergholm 2008-2014
 
-        return log2(2 * self.negativity(sys) + 1)
+        return np.log2(2 * self.negativity(sys) + 1)
 
 
     def scott(self, m):
@@ -1238,11 +1240,7 @@ class state(lmap):
 
         When m = 1 this is coincides with the Meyer-Wallach entanglement measure.
 
-        See [Love]_, [Scott]_, [MW]_.
-
-        .. [Love] P.J. Love et al., "A characterization of global entanglement", arXiv:quant-ph/0602143 (2006).
-        .. [Scott] A.J. Scott, "Multipartite entanglement, quantum-raise ValueError-correcting codes, and entangling power of quantum evolutions", PRA 69, 052330 (2004).
-        .. [MW] D.A. Meyer and N.R. Wallach, "Global entanglement in multiparticle systems", J. Math. Phys. 43, 4273 (2002).
+        See :cite:`Love`, :cite:`Scott`, :cite:`MW`.
         """
         # Jacob D. Biamonte 2008
         # Ville Bergholm 2008-2014
@@ -1262,7 +1260,7 @@ class state(lmap):
         Q = empty((N,))
         # Loop over all m-combinations of n subsystems, trace over everything except them.
         # reversed() fixes the order since we are actually looping over the complements.
-        for k, sys in enumerate(reversed(list(itertools.combinations(xrange(n), n-m)))):
+        for k, sys in enumerate(reversed(list(itertools.combinations(range(n), n-m)))):
             temp = self.ptrace(sys)  # trace over everything except S_k
             # NOTE: For pure states, tr(\rho_S^2) == tr(\rho_{\bar{S}}^2)
             temp = 1 - trace(np.linalg.matrix_power(temp.data, 2))
@@ -1274,11 +1272,11 @@ class state(lmap):
     def locc_convertible(self, t, sys):
         """LOCC convertibility of states.
 
-        For bipartite pure states s and t, returns true if s can be converted to t
+        For bipartite pure states s and t, returns True if s can be converted to t
         using local operations and classical communication (LOCC).
         sys is a vector of subsystems defining the partition.
 
-        See [NC]_, chapter 12.5.1
+        See :cite:`NC`, chapter 12.5.1
         """
         # Ville Bergholm 2010
 
@@ -1354,7 +1352,7 @@ class state(lmap):
 
             # add a colorbar
             cb = fig.colorbar(colormapper, ax = ax, ticks = linspace(-1, 1, 5))
-            cb.ax.set_yticklabels(['$-\pi$', '$-\pi/2$', '0', '$\pi/2$', '$\pi$'])
+            cb.ax.set_yticklabels([r'$-\pi$', r'$-\pi/2$', '0', r'$\pi/2$', r'$\pi$'])
 
             # the way it should work (using np.broadcast, ScalarMappable)
             #bars = ax.bar(range(N), s.prob(), color=c, cmap=cm.hsv, norm=whatever, align='center')
@@ -1388,7 +1386,7 @@ class state(lmap):
 
             # add a colorbar
             cb = fig.colorbar(pcol, ax = ax, ticks = linspace(-1, 1, 5))
-            cb.ax.set_yticklabels(['$-\pi$', '$-\pi/2$', '0', '$\pi/2$', '$\pi$'])
+            cb.ax.set_yticklabels([r'$-\pi$', r'$-\pi/2$', '0', r'$\pi/2$', r'$\pi$'])
 
             # the way it should work (using np.broadcast, ScalarMappable)
             #x, y = meshgrid(temp, temp)
@@ -1428,9 +1426,9 @@ class state(lmap):
 
            self.purity() <= 1   implies   norm(A, 'fro')  <= sqrt(D)
 
-           self.trace()   = 1   implies   A[0, 0, ..., 0] == sqrt(D)   
+           self.trace()   = 1   implies   A[0, 0, ..., 0] == 1
 
-        E.g. for a qubit system norm(A, 'fro') <= 2.
+        E.g. for a single-qubit system norm(A, 'fro') <= sqrt(2).
         """
         # Ville Bergholm 2009-2011
 
@@ -1468,7 +1466,7 @@ class state(lmap):
                 temp.append(k.to_op())
             arg = temp
 
-        return state(tensor(*arg))  # lmap.tensor
+        return state(lmap_tensor(*arg))
 
 
     @staticmethod
@@ -1504,153 +1502,14 @@ class state(lmap):
         return state(C * rho, dim)
 
 
-    @staticmethod
-    def test():
-        """Test script for the state class.
-        """
-        # Ville Bergholm 2008-2011
-
-        #for k in range(5):
-
-        # test the state constructor
-        s = state('101011')
-        s = state('2014', (3,2,3,5))
-        s = state(11, (3,5,2))
-        s = state(rand(5))
-        s = state(rand(6), (3,2))
-        s = state(rand(3,3))
-        s = state(rand(4,4), (2,2))
-        s = state('GHZ')
-        s = state('GHZ', (3,2,3))
-        s = state('W', (2,3,2))
-        s = state('Bell2')
-
-        # mixed states
-        dim = [2, 2]
-        rho1 = state(rand_positive(prod(dim)), dim)
-        rho2 = state(rand_positive(prod(dim)), dim)
-        U_r = rand_U(prod(dim))
-
-        dim = [2, 5, 3]
-        sigma1 = state(rand_positive(prod(dim)), dim)
-        U_s = rand_U(prod(dim))
-
-        # pure states
-        dim = [2, 2]
-        p = state(0, dim)
-
-        p1 = p.u_propagate(rand_SU(prod(dim)))
-        p2 = p.u_propagate(rand_SU(prod(dim)))
-        U_p = rand_U(prod(dim))
-
-
-        # TODO concurrence, fix_phase, kraus_propagate, locc_convertible, lognegativity, measure,
-        # negativity,
-
-        # generalized Bloch vectors.
-
-        temp = sigma1.bloch_vector();  D = prod(sigma1.dims())
-        assert_o((state.bloch_state(temp) -sigma1).norm(), 0, tol) # need to match
-        assert_o(norm(temp.imag), 0, tol) # correlation tensor is real
-        assert(norm(temp) -sqrt(D) <= tol)  # purity
-        assert_o(temp.flat[0], 1, sqrt(D))  # normalization
-
-        # fidelity, trace_dist
-
-        assert_o(fidelity(rho1, rho2), fidelity(rho2, rho1), tol) # symmetric
-        assert_o(trace_dist(rho1, rho2), trace_dist(rho2, rho1), tol)
-
-        assert_o(fidelity(sigma1, sigma1), 1, tol) # normalized to unity
-        assert_o(trace_dist(sigma1, sigma1), 0, tol) # distance measure
-
-        # unaffected by unitary transformations
-        assert_o(fidelity(rho1, rho2), fidelity(rho1.u_propagate(U_r), rho2.u_propagate(U_r)), tol)
-        assert_o(trace_dist(rho1, rho2), trace_dist(rho1.u_propagate(U_r), rho2.u_propagate(U_r)), tol)
-
-        # for pure states they're equivalent
-        assert_o(trace_dist(p1, p2) ** 2 +fidelity(p1, p2) ** 2, 1, tol)
-        # for mixed states, these inequalities hold
-        assert(sqrt(1 - fidelity(rho1, rho2) ** 2) - trace_dist(rho1, rho2) >= -tol)
-        assert(1 - fidelity(rho1, rho2) - trace_dist(rho1, rho2) <= tol)
-        # for a pure and a mixed state we get this inequality
-        assert(1 - fidelity(rho1, p1) ** 2 -trace_dist(rho1, p1) <= tol)
-
-
-        # entropy
-
-        assert_o(p1.entropy(), 0, tol) # zero for pure states
-        assert(sigma1.entropy() >= -tol) # nonnegative
-
-        # unaffected by unitary transformations
-        assert_o(sigma1.u_propagate(U_s).entropy(), sigma1.entropy(), tol)
-
-
-        # ptrace, ptranspose
-
-        rho_A = rho1.ptrace([1])
-        # trace of partial trace equals total trace
-        assert_o(rho1.trace(), rho_A.trace(), tol)
-        # partial trace over all subsystems equals total trace
-        assert_o(rho1.trace(), rho1.ptrace(range(rho1.subsystems())).trace(), tol)
-
-        rho_X = sigma1.ptrace([0, 2, 3])
-        assert_o(sigma1.trace(), rho_X.trace(), tol)
-        assert_o(sigma1.trace(), sigma1.ptrace(range(sigma1.subsystems())).trace(), tol)
-
-        rho_pt_B = rho1.ptranspose([1])
-        # two ptransposes cancel
-        assert_o(trace_dist(rho1, rho_pt_B.ptranspose([1])), 0, tol)
-        # ptranspose preserves trace
-        assert_o(rho1.trace(), rho_pt_B.trace(), tol)
-
-
-        # schmidt
-        
-        lambda1, u, v = p1.schmidt([0], full = True)
-        lambda2 = p1.schmidt([1])
-        # squares of schmidt coefficients sum up to unity
-        assert_o(norm(lambda1), 1, tol)
-        # both subdivisions have identical schmidt coefficients
-        assert_o(norm(lambda1-lambda2), 0, tol)
-
-        # decomposition is equal to the original matrix
-        temp = 0
-        for k in range(len(lambda1)):
-            temp += kron(lambda1[k]*u[:,k], v[:,k])
-        assert_o(norm(p1.data.ravel() - temp), 0, tol)
-
-        # squared schmidt coefficients equal eigenvalues of partial trace
-        r = state(randn(30) + 1j*randn(30), [5, 6]).normalize()
-        #x = r.schmidt((0,)) ** 2  # FIXME crashes ipython!
-        temp = r.ptrace([1])
-        y, dummy = eighsort(temp.data)
-        #assert_o(norm(x-y), 0, tol)
-
-
-        # reorder
-
-        dim = [2, 5, 1]
-        A = rand(dim[0], dim[0])
-        B = rand(dim[1], dim[1])
-        C = rand(dim[2], dim[2])
-
-        T1 = state(mkron(A, B, C), dim)
-        T2 = T1.reorder([2, 0, 1])
-        assert_o(norm(mkron(C, A, B) - T2.data), 0, tol)
-        T2 = T1.reorder([1, 0, 2])
-        assert_o(norm(mkron(B, A, C) - T2.data), 0, tol)
-        T2 = T1.reorder([2, 1, 0])
-        assert_o(norm(mkron(C, B, A) - T2.data), 0, tol)
-
-
 
 # wrappers
 
-def fidelity(s,t):
+def fidelity(s, t):
     """Wrapper for state.fidelity."""
     return s.fidelity(t)
 
 
-def trace_dist(s,t):
+def trace_dist(s, t):
     """Wrapper for state.trace_dist."""
     return s.trace_dist(t)
