@@ -95,7 +95,6 @@ Miscellaneous
    R_x
    R_y
    R_z
-   test
 """
 # Ville Bergholm 2008-2012
 
@@ -104,17 +103,18 @@ from __future__ import division, absolute_import, print_function, unicode_litera
 from copy import deepcopy
 
 import numpy as np
-import scipy as sp
-from numpy import array, mat, dtype, empty, zeros, ones, eye, prod, sqrt, exp, tanh, dot, sort, diag, trace, kron, pi, r_, c_, inf, isscalar, floor, ceil, log, log10, vdot
-from numpy.random import rand, randn, randint
+from numpy import (array, mat, dtype, empty, zeros, ones, eye, prod, sqrt, exp,
+    dot, sort, diag, trace, kron, pi, r_, c_, inf, isscalar, floor, ceil, log, log10, vdot)
+from numpy.random import rand, randn
 from numpy.linalg import qr, det, eigh, eigvalsh
 from scipy.linalg import expm, norm, svd, svdvals
 
-from .base import *
+from .base import sx, sy, sz, tol
 
 __all__ = ['assert_o', 'copy_memoize',
            'gcd', 'lcm', 'majorize',
-           'comm', 'acomm', 'mkron', 'rank', 'orth', 'nullspace', 'nullspace_hermitian', 'projector', 'eighsort', 'expv',
+           'comm', 'acomm', 'mkron', 'projector', 'eighsort', 'expv',
+           'rank', 'orth', 'nullspace', 'nullspace_hermitian',
            'rand_hermitian', 'rand_U', 'rand_SU', 'rand_U1', 'rand_pu', 'rand_positive', 'rand_SL', 
            'vec', 'inv_vec', 'lmul', 'rmul', 'lrmul', 'superop_lindblad', 'superop_fp',
            'angular_momentum', 'boson_ladder', 'fermion_ladder',
@@ -169,7 +169,7 @@ def gcd(a, b):
     Uses the Euclidean algorithm.
     """
     while b:
-        a, b = b, a%b
+        a, b = b, a % b
     return a
 
 
@@ -321,9 +321,7 @@ def expv(t, A, v, tol=1.0e-7, m=30, iteration='arnoldi'):
     error   total truncation error estimate
     hump    :math:`\max_{s \in [0, t]}  \| \exp(s A) \|`
 
-    Uses the sparse algorithm from [EXPOKIT]_.
-
-    .. [EXPOKIT] Sidje, R.B., "EXPOKIT: A Software Package for Computing Matrix Exponentials", ACM Trans. Math. Softw. 24, 130 (1998).
+    Uses the sparse algorithm from :cite:`EXPOKIT`.
     """
     # Ville Bergholm 2009-2012
 
@@ -508,7 +506,7 @@ def rand_hermitian(n):
     """
     # Ville Bergholm 2008-2009
 
-    H = (rand(n,n) - 0.5) +1j*(rand(n,n) - 0.5)
+    H = (rand(n, n) - 0.5) +1j*(rand(n, n) - 0.5)
     return H + H.conj().transpose() # make it Hermitian
 
 
@@ -518,15 +516,13 @@ def rand_U(n):
     Returns a random unitary n*n matrix.
     The matrix is random with respect to the Haar measure.
 
-    Uses the algorithm in [Mezzadri]_.
-
-    .. [Mezzadri] F.Mezzadri, "How to generate random matrices from the classical compact groups", Notices of the AMS 54, 592 (2007). arXiv.org:math-ph/0609050
+    Uses the algorithm in :cite:`Mezzadri`.
     """
     # Ville Bergholm 2005-2014
 
     # sample the Ginibre ensemble, p(Z(i,j)) == 1/pi * exp(-abs(Z(i,j))^2),
     # p(Z) == 1/pi^(n^2) * exp(-trace(Z'*Z))
-    Z = (randn(n,n) + 1j*randn(n,n)) / sqrt(2)
+    Z = (randn(n, n) + 1j*randn(n, n)) / sqrt(2)
 
     # QR factorization
     Q, R = qr(Z)
@@ -605,7 +601,7 @@ def rand_SL(n):
     """
     # Ville Bergholm 2011
 
-    S = randn(n,n) +1j*randn(n,n)
+    S = randn(n, n) +1j*randn(n, n)
     d = det(S) ** (1/n)
     return S/d
 
@@ -654,12 +650,13 @@ def inv_vec(v, dim=None):
 def lmul(L, q=None):
     r"""Superoperator equivalent for multiplying from the left.
 
-    .. math::
+    :param array L: matrix, shape (m, p)
+    :param int q: The shape of :math:`\rho` is (p, q). If q is not given, :math:`\rho` is assumed square.
 
-       L \rho = \text{inv\_vec}(\text{lmul}(L) \text{vec}(\rho))
+    Returns the superoperator that implements left multiplication
+    of a vectorized matrix :math:`\rho` by the matrix L.
 
-    Dimensions: L is [m, p], :math:`\rho` is [p, q].
-    If q is not given :math:`\rho` is assumed square.
+    .. math:: L \rho = \text{inv\_vec}(\text{lmul}(L) \text{vec}(\rho))
     """
     # Ville Bergholm 2009
 
@@ -671,12 +668,13 @@ def lmul(L, q=None):
 def rmul(R, p=None):
     r"""Superoperator equivalent for multiplying from the right.
 
-    .. math::
+    :param array R: matrix, shape (q, r)
+    :param int p: The shape of :math:`\rho` is (p, q). If p is not given, :math:`\rho` is assumed square.
 
-       \rho R = \text{inv\_vec}(\text{rmul}(R) \text{vec}(\rho))
+    Returns the superoperator that implements right multiplication
+    of a vectorized matrix :math:`\rho` by the matrix R.
 
-    Dimensions: :math:`\rho` is [p, q], R is [q, r].
-    If p is not given :math:`\rho` is assumed square.
+    .. math:: \rho R = \text{inv\_vec}(\text{rmul}(R) \text{vec}(\rho))
     """
     # Ville Bergholm 2009
 
@@ -688,9 +686,14 @@ def rmul(R, p=None):
 def lrmul(L, R):
     r"""Superoperator equivalent for multiplying both from left and right.
 
-    .. math::
+    :param array L: matrix, shape (m, p)
+    :param array R: matrix, shape (q, r)
 
-       L \rho R = \text{inv\_vec}(\text{lrmul}(L, R) \text{vec}(\rho))
+    Returns the superoperator that implements left multiplication
+    by the matrix L and right multiplication by the matrix R
+    of a vectorized matrix :math:`\rho`.
+
+    .. math:: L \rho R = \text{inv\_vec}(\text{lrmul}(L, R) \text{vec}(\rho))
     """
     # Ville Bergholm 2009-2011
 
@@ -721,12 +724,12 @@ def superop_lindblad(A, H=None):
         iH = 1j * H
 
     L = zeros(array(sh) ** 2, complex)
-    acomm = zeros(sh, complex)
+    ac = zeros(sh, complex)
     for k in A:
-        acomm += 0.5 * dot(k.conj().transpose(), k)
+        ac += 0.5 * dot(k.conj().transpose(), k)
         L += lrmul(k, k.conj().transpose()) 
 
-    L += lmul(-acomm -iH) +rmul(-acomm +iH)
+    L += lmul(-ac -iH) +rmul(-ac +iH)
     return L
 
 
@@ -818,25 +821,28 @@ def superop_fp(L, tol=None):
 # physical operators
 
 @copy_memoize
-def angular_momentum(n):
+def angular_momentum(d):
     r"""Angular momentum matrices.
 
-    (Jx, Jy, Jz) = angular_momentum(d)
+    :param int d: dimension, the corresponding angular momentum quantum number is j = (d-1)/2
+    :returns: 3-tuple of angular momentum matrices (Jx, Jy, Jz)
 
-    Returns a 3-tuple of angular momentum matrices :math:`\vec{J} / \hbar`
-    for the d-dimensional subspace defined by the
-    quantum number j == (d-1)/2.
+    Returns the dimensionless angular momentum matrices :math:`\vec{J} / \hbar`
+    for the d-dimensional subspace.
+
+    The angular momentum matrices fulfill the commutation relation
+    :math:`[J_x, J_y] = i J_z` and all its cyclic permutations.
     """
-    # Ville Bergholm 2009-2010
+    # Ville Bergholm 2009-2014
 
-    if n < 1:
+    if d < 1:
         raise ValueError('Dimension must be one or greater.')
 
-    j = (n - 1) / 2 # angular momentum quantum number, n == 2*j + 1
+    j = (d - 1) / 2 # angular momentum quantum number, d == 2*j + 1
     # raising operator in subspace J^2 = j*(j+1)
     m = j
-    Jplus = zeros((n, n))
-    for k in range(n-1):
+    Jplus = zeros((d, d))
+    for k in range(d-1):
         m -= 1
         Jplus[k, k+1] = sqrt(j*(j+1) -m*(m+1))
 
@@ -848,25 +854,34 @@ def angular_momentum(n):
 
 
 @copy_memoize
-def boson_ladder(n):
+def boson_ladder(d):
     r"""Bosonic ladder operators.
 
-    Returns the n-dimensional approximation of the bosonic
-    annihilation operator b for a single bosonic mode in the
-    number basis :math:`\{|0\rangle, |1\rangle, ..., |n-1\rangle\}`.
+    :param int d: truncation dimension
+    :returns: bosonic annihilation operator :math:`b`
+
+    Returns the d-dimensional approximation of the bosonic
+    annihilation operator :math:`b` for a single bosonic mode in the
+    number basis :math:`\{|0\rangle, |1\rangle, ..., |d-1\rangle\}`.
 
     The corresponding creation operator is :math:`b^\dagger`.
+    The (infinite-dimensional) bosonic annihilation and creation operators fulfill the commutation relation
+    :math:`[b, b^\dagger] = I`. Due to the d-dimensional basis truncation,
+    this does not hold for the last dimension of the b matrices returned by this function.
     """
     # Ville Bergholm 2009-2010
 
-    return diag(sqrt(range(1, n)), 1)
+    return diag(sqrt(range(1, d)), 1)
 
 
 @copy_memoize
 def fermion_ladder(n):
     r"""Fermionic ladder operators.
 
-    Returns a vector of fermionic annihilation operators for a
+    :param int n: number of fermionic modes
+    :returns: array of fermionic annihilation operators :math:`f_k`
+
+    Returns the fermionic annihilation operators for a
     system of n fermionic modes in the second quantization.
 
     The annihilation operators are built using the Jordan-Wigner
@@ -921,9 +936,9 @@ def R_nmr(theta, phi):
     Returns the one-qubit rotation by angle theta about the unit
     vector :math:`[\cos(\phi), \sin(\phi), 0]`, or :math:`\theta_\phi` in the NMR notation.
     """
-    # Ville Bergholm 2009
+    # Ville Bergholm 2009-2014
 
-    return expm(-1j * theta/2 * (cos(phi) * sx + sin(phi) * sy))
+    return expm(-1j * theta/2 * (np.cos(phi) * sx + np.sin(phi) * sy))
 
 
 def R_x(theta):
@@ -976,7 +991,7 @@ def spectral_decomposition(A):
     # combine projectors for degenerate eigenvalues
     a = [d[0]]
     P = [projector(v[:, 0])]
-    for k in range(1,len(d)):
+    for k in range(1, len(d)):
         temp = projector(v[:, k])
         if abs(d[k] - d[k-1]) > tol:
             # new eigenvalue, new projector
@@ -1011,18 +1026,18 @@ def gellmann(d):
     # diagonal
     ddd = zeros(d)
     ddd[0] = 1
-    x = 1 / sqrt(2);
+    x = 1 / sqrt(2)
     # iterate through the lower triangle
     n = 0
     for k in range(1, d):
         for j in range(0, k):
             # nondiagonal
-            G[n, k, j] = x;
-            G[n, j, k] = x;
+            G[n, k, j] = x
+            G[n, j, k] = x
             n += 1
 
-            G[n, k, j] =  1j * x;
-            G[n, j, k] = -1j * x;
+            G[n, k, j] =  1j * x
+            G[n, j, k] = -1j * x
             n += 1
         ddd[k] = -sum(ddd)
         G[n, :, :] = diag(ddd) / norm(ddd)
@@ -1035,7 +1050,7 @@ def gellmann(d):
 # TODO lazy evaluation/cache purging would be nice here to control memory usage
 tensorbasis_cache = {}
 def tensorbasis(n, d=None, get_locality=False):
-    """Hermitian tensor-product basis for End(H).
+    r"""Hermitian tensor-product basis for End(H).
 
     Returns a Hermitian basis for linear operators on the Hilbert space H
     which shares H's tensor product structure. The basis elements are tensor products
@@ -1107,20 +1122,20 @@ def tensorbasis(n, d=None, get_locality=False):
 # misc
 
 def op_list(G, dim):
-    """Operator consisting of k-local terms, given as a list.
+    r"""Operator consisting of k-local terms, given as a list.
+
+    :param list G: list of k-local operator terms
+    :param vector dim: vector of subsystem dimensions
+    :returns: operator O defined by the connection list
 
-    Returns the operator O defined by the connection list G.
-    dim is a vector of subsystem dimensions for O.
     G is a list of arrays, :math:`G = [c_1, c_2, ..., c_n]`,
     where each array :math:`c_i` corresponds to a term in O.
 
     An array that has 2 columns and k rows, :math:`c_i` = [(A1, s1), (A2, s2), ... , (Ak, sk)],
-    where Aj are operators and sj subsystem indices, corresponds to the
+    where Aj are arrays and sj subsystem indices, corresponds to the
     k-local term given by the tensor product
 
-    .. math::
-
-       A1_{s1} * A2_{s2} * ... * Ak_{sk}.
+    .. math:: A_1^{(s_1)} A_2^{(s_2)} \cdots A_k^{(s_k)}.
 
     The dimensions of all operators acting on subsystem sj must match dim[sj].
 
@@ -1140,10 +1155,10 @@ def op_list(G, dim):
 
     # TODO we could try to infer dim from the operators
     H = 0j
-    for k,spec in enumerate(G):
+    for k, spec in enumerate(G):
         a = -1  # last subsystem taken care of
         term = 1
-        for j,op in enumerate(spec):
+        for j, op in enumerate(spec):
             if len(op) != 2:
                 raise ValueError('Malformed local term {0} in spec {1}.'.format(j, k))
 
@@ -1174,15 +1189,18 @@ def qubits(n):
 
 
 def majorize(x, y):
-    """Majorization partial order of real vectors.
+    r"""Majorization partial order of real vectors.
+
+    :param vector x, y: real vectors, dimension d
+    :returns: True iff :math:`x \preceq y`
+
+    Returns True iff x is majorized by y, denoted by :math:`x \preceq y`. This is equivalent to
 
-    Returns true iff the real vector x is majorized by the real vector y,
-    denoted by :math:`x \preceq y`. This is equivalent to
     .. math::
 
-       \sum_{k=1}^n x^{\downarrow}_k \le \sum_{k=1}^n y^{\downarrow}_k` \quad \text{for all} \quad n \in \{1, 2, \ldots, d\},
+       \sum_{k=1}^n x^{\downarrow}_k \le \sum_{k=1}^n y^{\downarrow}_k \quad \text{for all} \quad n \in \{1, 2, \ldots, d\},
 
-    where d is the dimension of the vectors x and y, and :math:`x^{\downarrow}` is the vector x with the elements sorted in nonincreasing order.
+    where :math:`x^{\downarrow}` is the vector x with the elements sorted in nonincreasing order.
 
     :math:`x \preceq y` if and only if x is in the convex hull of all the coordinate permutations of y.
     """
@@ -1207,7 +1225,7 @@ def majorize(x, y):
 
 
 def mkron(*arg):
-    """This is how kron should work, dammit.
+    r"""This is how kron should work, dammit.
 
     Returns the tensor (Kronecker) product :math:`X = A \otimes B \otimes \ldots`
     """
@@ -1217,109 +1235,3 @@ def mkron(*arg):
     for A in arg:
         X = kron(X, A)
     return X
-
-
-def test():
-    """Test script for the utils module.
-    """
-    # Ville Bergholm 2009-2011
-
-    dim = 10
-
-    # expv
-    A = randn(dim, dim) + 1j * randn(dim, dim)
-    H = rand_hermitian(dim)
-    v = randn(dim) + 1j * randn(dim)
-
-    # arbitrary matrix with Arnoldi iteration
-    w, err, hump = expv(1, A, v, m = dim // 2)
-    assert_o(norm(w - dot(expm(1*A), v)), 0, 1e2*tol)
-    w, err, hump = expv(1, A, v, m = dim)  # force a happy breakdown
-    assert_o(norm(w - dot(expm(1*A), v)), 0, 1e2*tol)
-
-    # Hermitian matrix with Lanczos iteration
-    w, err, hump = expv(1, H, v, m = dim // 2, iteration = 'lanczos')
-    assert_o(norm(w - dot(expm(1*H), v)), 0, 1e2*tol)
-    w, err, hump = expv(1, H, v, m = dim, iteration = 'lanczos')
-    assert_o(norm(w - dot(expm(1*H), v)), 0, 1e2*tol)
-
-    # FIXME why does Lanczos work with nonhermitian matrices?
-    w, err, hump = expv(1, A, v, m = dim // 2, iteration = 'lanczos')
-    assert_o(norm(w - dot(expm(1*A), v)), 0, 1e2*tol)
-    w, err, hump = expv(1, A, v, m = dim, iteration = 'lanczos')
-    assert_o(norm(w - dot(expm(1*A), v)), 0, 1e2*tol)
-
-
-    dim = 5
-
-    # random matrices
-    H = mat(rand_hermitian(dim))
-    assert_o(norm(H - H.H), 0, tol) # hermitian
-
-    U = mat(rand_U(dim))
-    assert_o(norm(U * U.H -eye(dim)), 0, tol) # unitary
-    assert_o(norm(U.H * U -eye(dim)), 0, tol)
-
-    U = mat(rand_SU(dim))
-    assert_o(norm(U * U.H -eye(dim)), 0, tol) # unitary
-    assert_o(norm(U.H * U -eye(dim)), 0, tol)
-    assert_o(det(U), 1, tol) # det 1
-
-    rho = mat(rand_positive(dim))
-    assert_o(norm(rho - rho.H), 0, tol) # hermitian
-    assert_o(trace(rho), 1, tol) # trace 1
-    temp = eigvalsh(rho)
-    assert_o(norm(temp.imag), 0, tol) # real eigenvalues
-    assert_o(norm(temp - abs(temp)), 0, tol) # nonnegative eigenvalues
-
-
-    # superoperators
-    L = mat(rand_U(dim))
-    R = mat(rand_U(dim))
-    v = vec(array(rho))
-
-    assert_o(norm(rho -inv_vec(v)), 0, tol)
-    assert_o(norm(L*rho*R -inv_vec(dot(lrmul(L, R), v))), 0, tol)
-    assert_o(norm(L*rho -inv_vec(dot(lmul(L), v))), 0, tol)
-    assert_o(norm(rho*R -inv_vec(dot(rmul(R), v))), 0, tol)
-
-
-    # physical operators
-    J = angular_momentum(dim)
-    assert_o(norm(comm(J[0], J[1]) - 1j*J[2]), 0, tol)  # [Jx, Jy] == i Jz
-
-    a = mat(boson_ladder(dim))
-    temp = comm(a, a.H)
-    assert_o(norm(temp[:-1,:-1] - eye(dim-1)), 0, tol)  # [a, a'] == I  (truncated, so skip the last row/col!)
-
-    f = fermion_ladder(3)
-    # {f_j, f_k} = 0
-    # {f_j, f_k^\dagger} = I \delta_{jk}
-    for k in range(3):
-        temp = f[k].conj().transpose()
-        assert_o(norm(acomm(f[k], f[k])), 0, tol)
-        assert_o(norm(acomm(f[k], temp) -eye(8)), 0, tol)
-        for j in range(k):
-            assert_o(norm(acomm(f[j], f[k])), 0, tol)
-            assert_o(norm(acomm(f[j], temp)), 0, tol)
-
-    # SU(2) rotations
-
-
-    # spectral decomposition
-    E, P = spectral_decomposition(H)
-    temp = 0
-    for k in range(len(E)):
-        temp += E[k] * P[k]
-
-    assert_o(norm(temp - H), 0, tol)
-
-
-    # tensor bases
-
-
-    # majorization
-
-    # op_list
-
-    # plots
